# -*- coding: utf-8 -*-
import Crypto
from Crypto.PublicKey import RSA
from Crypto import Random
from hashlib import sha256

# Функция генерации ключей заданной длины (1024)
def generate_key(KEY_LENGTH=1024):
    random_value= Random.new().read
    keyPair=RSA.generate(KEY_LENGTH,random_value)
    return keyPair

# Генерация ключей для Алисы и Боба
bobKey=generate_key()
aliceKey=generate_key()

# Вывод на печать открытых ключей для Алисы и Боба
# Эти ключи должны быть обнародованы
alicePK=aliceKey.publickey()
bobPK=bobKey.publickey()

print "Alice's Public Key:", alicePK
print "Bob's Public Key:", bobPK

# Алиса хочет отправить сообщение Бобу
# Давайте создадим тестовое сообщение для Алисы
secret_message="Alice's secret message to Bob"
print "Message",  secret_message

# Функция генерации подписи
def generate_signature(key,message):
    message_hash=sha256(message).digest()
    signature=key.sign(message_hash,'')
    return signature

# Сгенерируем подпись для секретного сообщения
alice_sign=generate_signature(aliceKey,secret_message)

# Перед отправкой сообщения в сеть зашифруем его
# с использованием открытого ключа Боба
encrypted_for_bob  = bobPK.encrypt(secret_message, 32)  

# Боб расшифровывает секретное сообщение
# при помощи своего закрытого ключа
decrypted_message   = bobKey.decrypt(encrypted_for_bob)
print "Decrypted message:", decrypted_message

# Боб использует эту функцию для проверки подписи Алисы
# при помощи ее открытого ключа
def verify_signature(message,PublicKey,signature):
    message_hash=sha256(message).digest()
    verify = PublicKey.verify(message_hash,signature)
    return verify

# Боб проводит верификацию, используя расшифрованное
# сообщение и открытый ключ Алисы
print "Is alice's signature for decrypted message valid?", verify_signature(decrypted_message,alicePK, alice_sign)

# *********************************************
# *            The ECDSA Algorithm            *
# *********************************************
import ecdsa

# SECP256k1 это эллиптическая кривая биткойна
signingKey = ecdsa.SigningKey.generate(curve=ecdsa.SECP256k1)
# Получаем ключ верификации
verifyingKey = signingKey.get_verifying_key()

# Генерируем подпись сообщения
signature = signingKey.sign(b"signed message")

# Проверяем достоверность подписи
verifyingKey.verify(signature, b"signed message") 
# True => Подпись достоверна

# Проверяем достоверность подписи
assert verifyingKey.verify(signature, b"message") 
# Ошибка. Подпись недостоверна
